﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Text;

namespace TestProject
{
    /// <summary>
    /// 갤러리 헬퍼
    /// </summary>
    public static class GalleryHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 썸네일 비트맵 구하기 - GetThumbnailBitmap(sourceBitmap, targetWidth, targetHeight)

        /// <summary>
        /// 썸네일 비트맵 구하기
        /// </summary>
        /// <param name="sourceBitmap">소스 비트맵</param>
        /// <param name="targetWidth">타겟 너비</param>
        /// <param name="targetHeight">타겟 높이</param>
        /// <returns>썸네일 비트맵</returns>
        public static Bitmap GetThumbnailBitmap(Bitmap sourceBitmap, int targetWidth, int targetHeight)
        {
            Bitmap bitmap = new Bitmap(targetWidth, targetHeight);

            using(Graphics graphics = Graphics.FromImage(bitmap))
            {
                graphics.InterpolationMode  = InterpolationMode.HighQualityBicubic;
                graphics.CompositingQuality = CompositingQuality.HighQuality;
                graphics.SmoothingMode      = SmoothingMode.HighQuality;
                graphics.TextRenderingHint  = TextRenderingHint.AntiAlias;
                graphics.PixelOffsetMode    = PixelOffsetMode.HighQuality;

                graphics.Clear(Color.Transparent);

                int width;
                int height;

                if(sourceBitmap.Width < targetWidth && sourceBitmap.Height < targetHeight)
                {
                    width  = sourceBitmap.Width;
                    height = sourceBitmap.Height;
                }
                else
                {
                    if(sourceBitmap.Width > sourceBitmap.Height)
                    {
                        width  = targetWidth;
                        height = Convert.ToInt32(targetHeight * Convert.ToDouble(sourceBitmap.Height) / sourceBitmap.Width);
                    }
                    else
                    {
                        width  = Convert.ToInt32(targetWidth * Convert.ToDouble(sourceBitmap.Width) / sourceBitmap.Height); 
                        height = targetHeight;
                    }
                }

                int left = (targetWidth  - width ) / 2;
                int top  = (targetHeight - height) / 2;

                graphics.DrawImage(sourceBitmap, left, top, width, height);
            }

            return bitmap;
        }

        #endregion
        #region 썸네일 비트맵 구하기 - GetThumbnailBitmap(filePath)

        /// <summary>
        /// 썸네일 비트맵 구하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        /// <returns>썸네일 비트맵</returns>
        public static Bitmap GetThumbnailBitmap(string filePath)
        {
            using(Bitmap sourceBitmap = new Bitmap(filePath))
            {
                Bitmap thumbnailBitmap = GetThumbnailBitmap(sourceBitmap, 200, 200);

                return thumbnailBitmap;
            }
        }

        #endregion

        #region 행 인덱스 구하기 - GetRowIndex(columnCount, sequenceIndex)

        /// <summary>
        /// 행 인덱스 구하기
        /// </summary>
        /// <param name="columnCount">컬럼 수</param>
        /// <param name="sequenceIndex">순번 인덱스</param>
        /// <returns>행 인덱스</returns>
        public static int GetRowIndex(int columnCount, int sequenceIndex)
        {
            return sequenceIndex / columnCount;
        }

        #endregion
        #region 컬럼 인덱스 구하기 - GetColumnIndex(columnCount, sequenceIndex)

        /// <summary>
        /// 컬럼 인덱스 구하기
        /// </summary>
        /// <param name="columnCount">컬럼 수</param>
        /// <param name="sequenceIndex">순번 인덱스</param>
        /// <returns>컬럼 인덱스</returns>
        public static int GetColumnIndex(int columnCount, int sequenceIndex)
        {
            return sequenceIndex % columnCount;
        }

        #endregion
        #region 순번 인덱스 구하기 - GetSequenceIndex(columnCount, rowIndex, columnIndex)

        /// <summary>
        /// 순번 인덱스 구하기
        /// </summary>
        /// <param name="columnCount">컬럼 수</param>
        /// <param name="rowIndex">행 인덱스</param>
        /// <param name="columnIndex">컬럼 인덱스</param>
        /// <returns>순번 인덱스</returns>
        public static int GetSequenceIndex(int columnCount, int rowIndex, int columnIndex)
        {
            return columnCount * rowIndex + columnIndex;
        }

        #endregion

        #region 항목 그리기 - DrawItem(graphics, x, y, width, height, item)

        /// <summary>
        /// 항목 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="item">갤러리 항목</param>
        public static void DrawItem(Graphics graphics, int x, int y, int width, int height, GalleryItem item)
        {
            graphics.DrawRectangle(Pens.Black, x + 1, y + 1, width - 2, height - 2);

            if(item.Image != null)
            {
                graphics.DrawImage(item.Image, new Rectangle(x + 2, y + 2, width - 4, height - 4));
            }
        }

        #endregion
    }
}